@@ -1,10 +1,11 @@
 """
-Orange Canvas Application
-
 """
+import atexit
+import sys
 import os
 import argparse
-from typing import Optional
+import logging
+from typing import Optional, List, Sequence
 
 import AnyQt
 from AnyQt.QtWidgets import QApplication
@@ -12,6 +13,10 @@ from AnyQt.QtCore import (
     Qt, QUrl, QEvent, QSettings, QLibraryInfo, pyqtSignal as Signal
 )
 
+from orangecanvas.utils.after_exit import run_after_exit
+from orangecanvas.utils.asyncutils import get_event_loop
+from orangecanvas.gui.utils import macos_set_nswindow_tabbing
+
 
 def fix_qt_plugins_path():
     """
@@ -74,7 +79,7 @@ def fix_qt_plugins_path():
         except ValueError:
             pass
         if idx is None:
-            return
+            return None
         return os.path.join(prefix, *path_components[idx + 1:])
 
     newpath = maybe_match_prefix(
@@ -90,31 +95,70 @@ class CanvasApplication(QApplication):
     __args = None
 
     def __init__(self, argv):
+        CanvasApplication.__args, argv_ = self.parseArguments(argv)
+        ns = CanvasApplication.__args
         fix_qt_plugins_path()
-        if hasattr(Qt, "AA_EnableHighDpiScaling"):
+        self.__fileOpenUrls = []
+        self.__in_exec = False
+
+        if ns.enable_high_dpi_scaling \
+                and hasattr(Qt, "AA_EnableHighDpiScaling"):
             # Turn on HighDPI support when available
             QApplication.setAttribute(Qt.AA_EnableHighDpiScaling)
+        if ns.use_high_dpi_pixmaps \
+                and hasattr(Qt, "AA_UseHighDpiPixmaps"):
+            QApplication.setAttribute(Qt.AA_UseHighDpiPixmaps)
 
-        CanvasApplication.__args, argv_ = self.parse_style_arguments(argv)
-        if self.__args.style:
+        if ns.style:
             argv_ = argv_ + ["-style", self.__args.style]
+
         super().__init__(argv_)
+        # Make sure there is an asyncio event loop that runs on the
+        # Qt event loop.
+        _ = get_event_loop()
         argv[:] = argv_
         self.setAttribute(Qt.AA_DontShowIconsInMenus, True)
+        if hasattr(self, "styleHints"):
+            sh = self.styleHints()
+            if hasattr(sh, 'setShowShortcutsInContextMenus'):
+                # PyQt5.13 and up
+                sh.setShowShortcutsInContextMenus(True)
+        macos_set_nswindow_tabbing(False)
         self.configureStyle()
 
     def event(self, event):
         if event.type() == QEvent.FileOpen:
-            self.fileOpenRequest.emit(event.url())
+            if not self.__in_exec:
+                self.__fileOpenUrls.append(event.url())
+            else:
+                self.fileOpenRequest.emit(event.url())
         elif event.type() == QEvent.PolishRequest:
             self.configureStyle()
         return super().event(event)
 
+    def exec(self) -> int:
+        while self.__fileOpenUrls:
+            self.fileOpenRequest.emit(self.__fileOpenUrls.pop(0))
+        self.__in_exec = True
+        try:
+            return super().exec()
+        finally:
+            self.__in_exec = False
+
+    exec_ = exec
+
     @staticmethod
-    def parse_style_arguments(argv):
+    def argumentParser():
         parser = argparse.ArgumentParser()
         parser.add_argument("-style", type=str, default=None)
         parser.add_argument("-colortheme", type=str, default=None)
+        parser.add_argument("-enable-high-dpi-scaling", type=bool, default=True)
+        parser.add_argument("-use-high-dpi-pixmaps", type=bool, default=True)
+        return parser
+
+    @staticmethod
+    def parseArguments(argv):
+        parser = CanvasApplication.argumentParser()
         ns, rest = parser.parse_known_args(argv)
         if ns.style is not None:
             if ":" in ns.style:
@@ -147,3 +191,45 @@ class CanvasApplication(QApplication):
         if theme and theme in styles.colorthemes:
             palette = styles.colorthemes[theme]()
             QApplication.setPalette(palette)
+
+
+__restart_command: Optional[List[str]] = None
+
+
+def set_restart_command(cmd: Optional[Sequence[str]]):
+    """
+    Set or unset the restart command.
+
+    This command will be run after this process exits.
+
+    Pass cmd=None to unset the current command.
+    """
+    global __restart_command
+    log = logging.getLogger(__name__)
+    atexit.unregister(__restart)
+    if cmd is None:
+        __restart_command = None
+        log.info("Disabling application restart")
+    else:
+        __restart_command = list(cmd)
+        atexit.register(__restart)
+        log.info("Enabling application restart with: %r", cmd)
+
+
+def restart_command() -> Optional[List[str]]:
+    """Return the current set restart command."""
+    return __restart_command
+
+
+def restart_cancel() -> None:
+    set_restart_command(None)
+
+
+def default_restart_command():
+    """Return the default restart command."""
+    return [sys.executable, sys.argv[0]]
+
+
+def __restart():
+    if __restart_command:
+        run_after_exit(__restart_command)
