@@ -8,6 +8,7 @@ Scheme Editor Widget
 import io
 import logging
 import itertools
+import sys
 import unicodedata
 import copy
 import dictdiffer
@@ -16,12 +17,12 @@ from operator import attrgetter
 from urllib.parse import urlencode
 from contextlib import ExitStack, contextmanager
 from typing import (
-    List, Tuple, Optional, Container, Dict, Any, Iterable, Generator
+    List, Tuple, Optional, Container, Dict, Any, Iterable, Generator, Sequence
 )
 
 from AnyQt.QtWidgets import (
-    QWidget, QVBoxLayout, QInputDialog, QMenu, QAction, QActionGroup,
-    QUndoStack, QUndoCommand, QGraphicsItem, QGraphicsTextItem,
+    QWidget, QVBoxLayout, QMenu, QAction, QActionGroup,
+    QUndoStack, QGraphicsItem, QGraphicsTextItem,
     QFormLayout, QComboBox, QDialog, QDialogButtonBox, QMessageBox, QCheckBox,
     QGraphicsSceneDragDropEvent, QGraphicsSceneMouseEvent,
     QGraphicsSceneContextMenuEvent, QGraphicsView, QGraphicsScene,
@@ -32,17 +33,20 @@ from AnyQt.QtGui import (
     QWhatsThisClickedEvent, QKeyEvent, QPalette
 )
 from AnyQt.QtCore import (
-    Qt, QObject, QEvent, QSignalMapper, QCoreApplication, QPoint, QPointF,
+    Qt, QObject, QEvent, QSignalMapper, QCoreApplication, QPointF,
     QMimeData, Slot)
 from AnyQt.QtCore import pyqtProperty as Property, pyqtSignal as Signal
 
 from orangecanvas.document.commands import UndoCommand
+from .interactions import DropHandler
 from ..registry import WidgetDescription, WidgetRegistry
 from .suggestions import Suggestions
 from .usagestatistics import UsageStatistics
 from ..registry.qt import whats_this_helper, QtWidgetRegistry
 from ..gui.quickhelp import QuickHelpTipEvent
-from ..gui.utils import message_information, disabled
+from ..gui.utils import (
+    message_information, disabled, clipboard_has_format, clipboard_data
+)
 from ..scheme import (
     scheme, signalmanager, Scheme, SchemeNode, SchemeLink,
     BaseSchemeAnnotation, SchemeTextAnnotation, WorkflowEvent
@@ -55,7 +59,8 @@ from ..canvas.items.annotationitem import Annotation as AnnotationItem
 from . import interactions
 from . import commands
 from . import quickmenu
-from ..utils import findf
+from ..utils import findf, UNUSED
+from ..utils.qinvoke import connect_with_context
 
 Pos = Tuple[float, float]
 RuntimeState = signalmanager.SignalManager.State
@@ -155,7 +160,7 @@ class SchemeEditWidget(QWidget):
 
         self.__quickMenuTriggers = SchemeEditWidget.SpaceKey | \
                                    SchemeEditWidget.DoubleClicked
-        self.__emptyClickButtons = 0
+        self.__emptyClickButtons = Qt.NoButton
         self.__channelNamesVisible = True
         self.__nodeAnimationEnabled = True
         self.__possibleSelectionHandler = None
@@ -185,6 +190,8 @@ class SchemeEditWidget(QWidget):
         # list of annotations when set to a clean state
         self.__cleanAnnotations = []
 
+        self.__dropHandlers = ()  # type: Sequence[DropHandler]
+
         self.__editFinishedMapper = QSignalMapper(self)
         self.__editFinishedMapper.mapped[QObject].connect(
             self.__onEditingFinished
@@ -343,9 +350,9 @@ class SchemeEditWidget(QWidget):
             enabled=False
         )
 
-        shortcuts = [Qt.Key_Backspace,
-                     Qt.Key_Delete,
-                     Qt.ControlModifier + Qt.Key_Backspace]
+        shortcuts = [QKeySequence(Qt.Key_Backspace),
+                     QKeySequence(Qt.Key_Delete),
+                     QKeySequence(Qt.ControlModifier | Qt.Key_Backspace)]
 
         self.__removeSelectedAction.setShortcuts(shortcuts)
 
@@ -357,6 +364,12 @@ class SchemeEditWidget(QWidget):
             shortcut=QKeySequence(Qt.Key_F2),
             enabled=False
         )
+        if sys.platform == "darwin":
+            self.__renameAction.setShortcuts([
+                QKeySequence(Qt.Key_F2),
+                QKeySequence(Qt.Key_Enter),
+                QKeySequence(Qt.Key_Return)
+            ])
 
         self.__helpAction = QAction(
             self.tr("Help"), self,
@@ -395,7 +408,7 @@ class SchemeEditWidget(QWidget):
             self.tr("Duplicate"), self,
             objectName="duplicate-action",
             enabled=False,
-            shortcut=QKeySequence(Qt.ControlModifier + Qt.Key_D),
+            shortcut=QKeySequence(Qt.ControlModifier | Qt.Key_D),
             triggered=self.__duplicateSelected,
         )
 
@@ -403,7 +416,7 @@ class SchemeEditWidget(QWidget):
             self.tr("Copy"), self,
             objectName="copy-action",
             enabled=False,
-            shortcut=QKeySequence(Qt.ControlModifier + Qt.Key_C),
+            shortcut=QKeySequence(Qt.ControlModifier | Qt.Key_C),
             triggered=self.__copyToClipboard,
         )
 
@@ -411,7 +424,7 @@ class SchemeEditWidget(QWidget):
             self.tr("Paste"), self,
             objectName="paste-action",
             enabled=clipboard_has_format(MimeTypeWorkflowFragment),
-            shortcut=QKeySequence(Qt.ControlModifier + Qt.Key_V),
+            shortcut=QKeySequence(Qt.ControlModifier | Qt.Key_V),
             triggered=self.__pasteFromClipboard,
         )
         QApplication.clipboard().dataChanged.connect(
@@ -482,7 +495,7 @@ class SchemeEditWidget(QWidget):
         self.__raiseWidgetsAction = QAction(
             self.tr("Bring Widgets to Front"), self,
             objectName="bring-widgets-to-front-action",
-            shortcut=QKeySequence(Qt.ControlModifier + Qt.Key_Down),
+            shortcut=QKeySequence(Qt.ControlModifier | Qt.Key_Down),
             shortcutContext=Qt.WindowShortcut,
         )
         self.__raiseWidgetsAction.triggered.connect(self.__raiseToFont)
@@ -906,6 +919,9 @@ class SchemeEditWidget(QWidget):
             self.__scene.set_registry(registry)
             self.__quickMenu = None
 
+    def registry(self):
+        return self.__registry
+
     def quickMenu(self):
         # type: () -> quickmenu.QuickMenu
         """
@@ -1232,19 +1248,24 @@ class SchemeEditWidget(QWidget):
     def editNodeTitle(self, node):
         # type: (SchemeNode) -> None
         """
-        Edit (rename) the `node`'s title. Opens an input dialog.
+        Edit (rename) the `node`'s title.
         """
-        name, ok = QInputDialog.getText(
-            self, self.tr("Rename"),
-            self.tr("Enter a new name for the '%s' widget") % node.title,
-            text=node.title
-        )
+        self.__view.setFocus(Qt.OtherFocusReason)
+        scene = self.__scene
+        item = scene.item_for_node(node)
+        item.editTitle()
 
-        if ok and self.__scheme is not None:
+        def commit():
+            name = item.title()
+            if name == node.title:
+                return  # pragma: no cover
             self.__undoStack.push(
                 commands.RenameNodeCommand(self.__scheme, node, node.title,
                                            name)
             )
+        connect_with_context(
+            item.titleEditingFinished, self, commit
+        )
 
     def __onCleanChanged(self, clean):
         # type: (bool) -> None
@@ -1252,6 +1273,12 @@ class SchemeEditWidget(QWidget):
             self.setWindowModified(not clean)
             self.modificationChanged.emit(not clean)
 
+    def setDropHandlers(self, dropHandlers: Sequence[DropHandler]) -> None:
+        """
+        Set handlers for drop events onto the workflow view.
+        """
+        self.__dropHandlers = tuple(dropHandlers)
+
     def changeEvent(self, event):
         # type: (QEvent) -> None
         if event.type() == QEvent.FontChange:
@@ -1262,51 +1289,61 @@ class SchemeEditWidget(QWidget):
 
         super().changeEvent(event)
 
+    def __lookup_registry(self, qname: str) -> Optional[WidgetDescription]:
+        if self.__registry is not None:
+            try:
+                return self.__registry.widget(qname)
+            except KeyError:
+                pass
+        return None
+
+    def __desc_from_mime_data(self, data: QMimeData) -> Optional[WidgetDescription]:
+        MIME_TYPES = [
+            "application/vnd.orange-canvas.registry.qualified-name",
+            # A back compatible misspelling
+            "application/vnv.orange-canvas.registry.qualified-name",
+        ]
+        for typ in MIME_TYPES:
+            if data.hasFormat(typ):
+                qname_bytes = bytes(data.data(typ).data())
+                try:
+                    qname = qname_bytes.decode("utf-8")
+                except UnicodeDecodeError:
+                    return None
+                return self.__lookup_registry(qname)
+        return None
+
     def eventFilter(self, obj, event):
         # type: (QObject, QEvent) -> bool
         # Filter the scene's drag/drop events.
-        MIME_TYPE = "application/vnv.orange-canvas.registry.qualified-name"
         if obj is self.scene():
             etype = event.type()
             if etype == QEvent.GraphicsSceneDragEnter or \
                     etype == QEvent.GraphicsSceneDragMove:
                 assert isinstance(event, QGraphicsSceneDragDropEvent)
-                mime_data = event.mimeData()
                 drop_target = None
-                if mime_data.hasFormat(MIME_TYPE):
-                    qname = bytes(mime_data.data(MIME_TYPE)).decode("ascii")
-                    try:
-                        desc = self.__registry.widget(qname)
-                    except KeyError:
-                        pass
-                    else:
-                        item = self.__scene.item_at(event.scenePos(), items.LinkItem)
-                        link = self.scene().link_for_item(item) if item else None
-                        if link is not None and can_insert_node(desc, link):
-                            drop_target = item
-                            drop_target.setHoverState(True)
+                desc = self.__desc_from_mime_data(event.mimeData())
+                if desc is not None:
+                    item = self.__scene.item_at(event.scenePos(), items.LinkItem)
+                    link = self.scene().link_for_item(item) if item else None
+                    if link is not None and can_insert_node(desc, link):
+                        drop_target = item
+                        drop_target.setHoverState(True)
                     event.acceptProposedAction()
-                else:
-                    event.ignore()
-
                 if self.__dropTarget is not None and \
                         self.__dropTarget is not drop_target:
                     self.__dropTarget.setHoverState(False)
                 self.__dropTarget = drop_target
-                return True
+                if desc is not None:
+                    return True
             elif etype == QEvent.GraphicsSceneDragLeave:
                 if self.__dropTarget is not None:
                     self.__dropTarget.setHoverState(False)
                     self.__dropTarget = None
             elif etype == QEvent.GraphicsSceneDrop:
                 assert isinstance(event, QGraphicsSceneDragDropEvent)
-                data = event.mimeData()
-                qname = data.data(MIME_TYPE)
-                try:
-                    desc = self.__registry.widget(bytes(qname).decode("utf-8"))
-                except KeyError:
-                    log.error("Unknown qualified name '%s'", qname)
-                else:
+                desc = self.__desc_from_mime_data(event.mimeData())
+                if desc is not None:
                     statistics = self.usageStatistics()
                     pos = event.scenePos()
                     item = self.__scene.item_at(event.scenePos(), items.LinkItem)
@@ -1318,8 +1355,16 @@ class SchemeEditWidget(QWidget):
                     else:
                         statistics.begin_action(UsageStatistics.ToolboxDrag)
                         self.createNewNode(desc, position=(pos.x(), pos.y()))
-                return True
+                    return True
 
+            if etype == QEvent.GraphicsSceneDragEnter:
+                return self.sceneDragEnterEvent(event)
+            elif etype == QEvent.GraphicsSceneDragMove:
+                return self.sceneDragMoveEvent(event)
+            elif etype == QEvent.GraphicsSceneDragLeave:
+                return self.sceneDragLeaveEvent(event)
+            elif etype == QEvent.GraphicsSceneDrop:
+                return self.sceneDropEvent(event)
             elif etype == QEvent.GraphicsSceneMousePress:
                 self.__pasteOrigin = event.scenePos()
                 return self.sceneMousePressEvent(event)
@@ -1370,6 +1415,22 @@ class SchemeEditWidget(QWidget):
             event.accept()
             return True
         any_item = scene.item_at(pos)
+        # start node name edit on selected clicked
+        if sys.platform == "darwin" \
+                and event.button() == Qt.LeftButton \
+                and isinstance(any_item, items.nodeitem.GraphicsTextEdit) \
+                and isinstance(any_item.parentItem(), items.NodeItem):
+            node = scene.node_for_item(any_item.parentItem())
+            selected = self.selectedNodes()
+            if node in selected:
+                # deselect all other elements except the node item
+                # and start the edit
+                for selected_node in selected:
+                    selected_node_item = scene.item_for_node(selected_node)
+                    selected_node_item.setSelected(selected_node is node)
+                self.editNodeTitle(node)
+                return True
+
         if not any_item:
             self.__emptyClickButtons |= event.button()
 
@@ -1510,9 +1571,6 @@ class SchemeEditWidget(QWidget):
             handler = interactions.NewNodeAction(self)
             searchText = event.text()
 
-            # TODO: set the search text to event.text() and set focus on the
-            # search line
-
         if handler is not None:
             # Control + Backspace (remove widget action on Mac OSX) conflicts
             # with the 'Clear text' action in the search widget (there might
@@ -1596,6 +1654,31 @@ class SchemeEditWidget(QWidget):
 
         return False
 
+    def sceneDragEnterEvent(self, event: QGraphicsSceneDragDropEvent) -> bool:
+        UNUSED(event)
+        delegate = self._userInteractionHandler()
+        if delegate is not None:
+            return False
+
+        handler = interactions.DropAction(self, dropHandlers=self.__dropHandlers)
+        self._setUserInteractionHandler(handler)
+        return False
+
+    def sceneDragMoveEvent(self, event: QGraphicsSceneDragDropEvent) -> bool:
+        UNUSED(event)
+        return False
+
+    def sceneDragLeaveEvent(self, event: QGraphicsSceneDragDropEvent) -> bool:
+        UNUSED(event)
+        return False
+
+    def sceneDropEvent(self, event: QGraphicsSceneDragDropEvent) -> bool:
+        UNUSED(event)
+        return False
+
+    def _userInteractionHandler(self):
+        return self.__scene.user_interaction_handler
+
     def _setUserInteractionHandler(self, handler):
         # type: (Optional[interactions.UserInteraction]) -> None
         """
@@ -1630,6 +1713,7 @@ class SchemeEditWidget(QWidget):
         annotations = self.selectedAnnotations()
         links = self.selectedLinks()
 
+        self.__renameAction.setEnabled(len(nodes) == 1)
         self.__openSelectedAction.setEnabled(bool(nodes))
         self.__removeSelectedAction.setEnabled(
             bool(nodes or annotations or links)
@@ -1941,7 +2025,7 @@ class SchemeEditWidget(QWidget):
 
         view = self.view()
         try:
-            action = menu.exec_(view.mapToGlobal(view.mapFromScene(QPointF(x, y))))
+            action = menu.exec(view.mapToGlobal(view.mapFromScene(QPointF(x, y))))
         finally:
             menu.setFilterFunc(None)
 
@@ -2292,7 +2376,7 @@ class SaveWindowGroup(QDialog):
         layout.addLayout(form)
         self._combobox = cb = QComboBox(
             editable=True, minimumContentsLength=16,
-            sizeAdjustPolicy=QComboBox.AdjustToMinimumContentsLength,
+            sizeAdjustPolicy=QComboBox.AdjustToMinimumContentsLengthWithIcon,
             insertPolicy=QComboBox.NoInsert,
         )
         cb.currentIndexChanged.connect(self.__currentIndexChanged)
@@ -2463,33 +2547,6 @@ def copy_link(link, source=None, sink=None):
         properties=copy.deepcopy(link.properties))
 
 
-def clipboard_has_format(mimetype):
-    # type: (str) -> bool
-    """Does the system clipboard contain data for mimetype?"""
-    cb = QApplication.clipboard()
-    if cb is None:
-        return False
-    mime = cb.mimeData()
-    if mime is None:
-        return False
-    return mime.hasFormat(mimetype)
-
-
-def clipboard_data(mimetype):
-    # type: (str) -> Optional[bytes]
-    """Return the binary data of the system clipboard for mimetype."""
-    cb = QApplication.clipboard()
-    if cb is None:
-        return None
-    mime = cb.mimeData()
-    if mime is None:
-        return None
-    if mime.hasFormat(mimetype):
-        return bytes(mime.data(mimetype))
-    else:
-        return None
-
-
 def nodes_top_left(nodes):
     # type: (List[SchemeNode]) -> QPointF
     """Return the top left point of bbox containing all the node positions."""
