@@ -37,6 +37,9 @@ from ..registry import WidgetDescription, InputSignal, OutputSignal
 
 log = logging.getLogger(__name__)
 
+# protocol v4 is supported since Python 3.4, protocol v5 since Python 3.8
+PICKLE_PROTOCOL = 4
+
 
 class UnknownWidgetDefinition(Exception):
     pass
@@ -318,30 +321,39 @@ def parse_ows_etree_v_2_0(tree):
     )
 
 
+class InvalidFormatError(ValueError):
+    pass
+
+
+class UnsupportedFormatVersionError(ValueError):
+    pass
+
+
 def parse_ows_stream(stream):
     # type: (Union[AnyStr, IO]) -> _scheme
     doc = parse(stream)
     scheme_el = doc.getroot()
     if scheme_el.tag != "scheme":
-        raise ValueError(
+        raise InvalidFormatError(
             "Invalid Orange Workflow Scheme file"
         )
     version = scheme_el.get("version", None)
     if version is None:
         # Check for "widgets" tag - old Orange<2.7 format
         if scheme_el.find("widgets") is not None:
-            raise ValueError(
+            raise UnsupportedFormatVersionError(
                 "Cannot open Orange Workflow Scheme v1.0. This format is no "
                 "longer supported"
             )
         else:
-            raise ValueError(
+            raise InvalidFormatError(
                 "Invalid Orange Workflow Scheme file (missing version)."
             )
     if version in {"2.0", "2.1"}:
         return parse_ows_etree_v_2_0(doc)
     else:
-        raise ValueError()
+        raise UnsupportedFormatVersionError(
+            f"Unsupported format version {version}")
 
 
 def resolve_replaced(scheme_desc: _scheme, registry: WidgetRegistry) -> _scheme:
@@ -715,14 +727,16 @@ def dumps(obj, format="literal", prettyprint=False, pickle_fallback=False):
                         exc_info=True)
 
     elif format == "pickle":
-        return base64.encodebytes(pickle.dumps(obj)).decode('ascii'), "pickle"
+        return base64.encodebytes(pickle.dumps(obj, protocol=PICKLE_PROTOCOL)). \
+                   decode('ascii'), "pickle"
 
     else:
         raise ValueError("Unsupported format %r" % format)
 
     if pickle_fallback:
         log.warning("Using pickle fallback")
-        return base64.encodebytes(pickle.dumps(obj)).decode('ascii'), "pickle"
+        return base64.encodebytes(pickle.dumps(obj, protocol=PICKLE_PROTOCOL)). \
+                   decode('ascii'), "pickle"
     else:
         raise Exception("Something strange happened.")
 
